
/******************************************************************
  (C) Copyright 2001 by UNITED BYTES
   
  web:  http://www.unitedbytes.de
        http://www.unitedbytes.org
		http://www.unitedbytes.net
  mail: info@unitedbytes.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, 
  Boston, MA  02111-1307, USA.

  *****************************************************************

  Program:      ubNetwork
  File:         sources/cub_pop_client.cpp
  Author:       Andreas Loeffler
  Date:         Mon, 10.04.2001
  Version:      1.0
  Description:  Pop client implementation (RFC 1939) 

  -----------------------------------------------------------------

  Changes:		-
  Date:			-
  Description:	-

  *****************************************************************/

#include "cub_pop_client.h"

CUB_popClient::CUB_popClient ()
{
	Shutdown ();
}

CUB_popClient::~CUB_popClient ()
{
	Shutdown ();
}

void CUB_popClient::Shutdown ()
{
	// Clear variables
	m_lNumMessages    = 0;
	m_lSizeOfMessages = 0;

	// Send quit message
	SendCommand ("QUIT");
}

bool CUB_popClient::ConnectToPOP3 (CUB_popClientInfo *pClientInfo)
{
	int  iError = 0;
	char cBuffer [CUB_POP_MAX_MESSAGE];

	// Check for valid data
	if (!pClientInfo) {
		sprintf (m_cErrorText, "CUB_popClient->ConnectToPOP3: Invalid connection parameters given!");
		iError = 1;
	}
	else {
		// Set up the parameters
		if (!Setup (pClientInfo->m_cPopServer, pClientInfo->m_lServerPort)) {
			iError = 1;
		}
		else {
			// Initialize the client
			if (!Initialize ()) {
				iError = 1;
			}
			else {
				// Connect to the server!
				if (!Connect ()) {
					iError = 1;
				}
			}
		}
	}

	// Still no error occured?
	if (iError == 0)
	{
		////////////////////////////////////////////////////////////////////////////////////////
		// Send login information
		////////////////////////////////////////////////////////////////////////////////////////

		// Eat (=hide) welcome message
		ReceiveServerMessage (cBuffer, CUB_POP_MAX_MESSAGE);

		// Send mailbox name
		sprintf (cBuffer, "USER %s", pClientInfo->m_cMailboxName);
		SendCommand (cBuffer);

		// Await response
		ReceiveServerMessage (cBuffer, CUB_POP_MAX_MESSAGE);
		
		if (!CommandValid (cBuffer)) {
			sprintf (m_cErrorText, "User/Mailbox name not correct!");
			iError = 1;
		}
		else {
			// Send password
			sprintf (cBuffer, "PASS %s", pClientInfo->m_cPassword);
			SendCommand (cBuffer);

			// Await response
			ReceiveServerMessage (cBuffer, CUB_POP_MAX_MESSAGE);

			if (!CommandValid (cBuffer)) {
				sprintf (m_cErrorText, "Password name not correct or logged in too frequently!");
				iError = 1;
			}
			else {
				// Send request for statistics
				SendCommand ("STAT");

				// Read number and size of all messages in mailbox
				ReceiveServerMessage (cBuffer, CUB_POP_MAX_MESSAGE);

				if (!CommandValid (cBuffer)) {
					sprintf (m_cErrorText, "Cannot retrieve number of mails!");
					iError = 1;
				}
				else {
					// Extract the numbers
					sscanf (cBuffer, "+OK %ld %ld", &m_lNumMessages, &m_lSizeOfMessages);
				}
			}
		}
	}

	if (iError != 0)
		return false;

	return true;
}

bool CUB_popClient::CommandValid (char *cCommand)
{
	////////////////////////////////////////////////////////////////////////////////////////
	// Checks for +OK or -ERR at the beginning
	////////////////////////////////////////////////////////////////////////////////////////

	int iError = 0;
	char cBuffer [16];

	if (!cBuffer) {
		sprintf (m_cErrorText, "CUB_popClient->CommandValid: Buffer pointer invalid or too small!");
		iError = 1;
	}
	else {
		// Clear buffer
		memset (cBuffer, 0, sizeof (cBuffer));

		// Get the +OK (maybe)
		memcpy (cBuffer, cCommand, 3);

		if (!strcmp (cBuffer, "+OK"))
			return true;
		else 
			return false;
	}

	if (iError != 0)
		return false;

	return true;
}

long CUB_popClient::ReceiveServerMessage (char *cBuffer, long lBufLen)
{
	////////////////////////////////////////////////////////////////////////////////////////
	// Receives a message stream from the POP3 Server
	////////////////////////////////////////////////////////////////////////////////////////

	int		iError   = 0;
	int		iReturn  = 0;
	long	lCounter = 0;
	char	cChar    = 0;

	if ((!cBuffer) || lBufLen < CUB_POP_MAX_MESSAGE) {
		sprintf (m_cErrorText, "CUB_popClient->ReceiveServerMessage: Buffer pointer invalid or too small!");
		iError = 1;
	}
	else {
		memset (cBuffer, 0, lBufLen);

		if (m_iInitLevel < CUB_NS_CONNECTED) {
			sprintf (m_cErrorText, "CUB_popClient->ReceiveServerMessage: Client not connected yet!");
			iError = 1;
		}
	}

	if (iError == 0)
	{
		while (true)
		{
			iReturn = m_Socket.Receive (&cChar, 1);

			if (iReturn <= 0)
				break;

			cBuffer[lCounter] = cChar;

			lCounter ++;

			if (cChar == '\n')
				break;
		}	   
	}

	// Remove the CRLF from the buffer
	cBuffer [lCounter - 1] = '\0';

	if (iError != 0)
		return 0;

	return lCounter;
}

bool CUB_popClient::SendServerMessage (char *cBuffer, long lBufLen)
{
	int		iError    = 0;
	long	lCounter  = 0;
	long	lNumChars = 0;

	if (!cBuffer) {
		sprintf (m_cErrorText, "CUB_popClient->SendServerMessage: Buffer pointer invalid!");
		iError = 1;
	}
	else {
		if (m_iInitLevel < CUB_NS_CONNECTED) {
			sprintf (m_cErrorText, "CUB_popClient->SendServerMessage: Client not connected yet!");
			iError = 1;
		}
	}

	// Set an CRLF at the end of the line
	strcat (cBuffer, "\n");

	// Count the chars in the string
	lNumChars = strlen (cBuffer);

	if (iError == 0)
	{
		while (lCounter < lNumChars)
		{
			lCounter += m_Socket.Send (cBuffer, lNumChars);
		}	   
	}

	if (iError != 0)
		return false;

	return true;	
}

bool CUB_popClient::SendCommand (char *cCommand)
{
	////////////////////////////////////////////////////////////////////////////////////////
	// Send a command to the server
	////////////////////////////////////////////////////////////////////////////////////////

	int	 iError = 0;
	char cBuffer [CUB_POP_MAX_MESSAGE];

	if (!cBuffer) {
		sprintf (m_cErrorText, "CUB_popClient->SendCommand: Buffer pointer invalid!");
		iError = 1;
	}
	else {
		if (m_iInitLevel < CUB_NS_CONNECTED) {
			sprintf (m_cErrorText, "CUB_popClient->SendCommand: Client not connected yet!");
			iError = 1;
		}
	}

	if (iError == 0)
	{
		sprintf (cBuffer, "%s", cCommand);
		return SendServerMessage (cBuffer, CUB_POP_MAX_MESSAGE);
	}

	return false;
}

bool CUB_popClient::MessageManager ()
{
	////////////////////////////////////////////////////////////////////////////////////////
	// Scans all incoming messages
	////////////////////////////////////////////////////////////////////////////////////////

	int  iError = 0;
	char cBuffer [CUB_POP_MAX_MESSAGE];

	if (m_iInitLevel < CUB_NS_CONNECTED) {
		sprintf (m_cErrorText, "CUB_popClient->Refresh: Client not connected yet!");
		iError = 1;
	}
	else {
		// Something found in buffer?
		if (m_Socket.CanReceive ()) 
		{
			// Receive message from server
			if (!ReceiveServerMessage (cBuffer, 512)) {
				iError = 1;
			}
			else {
				if (strcmp (cBuffer, "")) 
					printf ("%s\n", cBuffer);
			}

		}

	}

	return false;
}

long CUB_popClient::GetNumMessages ()
{
	return m_lNumMessages;
}

long CUB_popClient::GetSizeOfMessages ()
{
	return m_lSizeOfMessages;
}

